function FTreeClass()
{
   this.init();
   this._x = Math.round(this._x);
   this._y = Math.round(this._y);
   this.itemSymbol = "FTreeSelectableItemSymbol";
   super.setDataProvider(new FTreeDataProviderClass());
   if(this.textStyle == undefined)
   {
      this.textStyle = new TextFormat();
   }
   this.textStyle.font = "Arial";
   this.textStyle.size = 12;
   this.styleTable.embedFonts = true;
   this.permaScrollBar = true;
   this.controller = this;
   this.mcCount = 3;
   this.boundingBox_mc.gotoAndStop(1);
   this.width = this._width;
   this.height = this._height;
   this._yscale = this._xscale = 100;
   this.setSize(this.width,this.height);
   this.selected = new Array();
   this.defaultNodeIconSymbolName = "nodeIconSymbol";
   this.drawConnectors_bool = true;
   if(this.changeHandler.length > 0)
   {
      this.setChangeHandler(this.changeHandler);
   }
   this.treewidth = -1;
}
FTreeClass.prototype = new FScrollSelectListClass();
Object.registerClass("FTreeSymbol",FTreeClass);
FTreeClass.prototype.addNode = function(parent, child)
{
   this.addNodeAt(parent,child,parent.length);
};
FTreeClass.prototype.addNodeAt = function(parent, child, index)
{
   if(this.enable)
   {
      parent.addNodeAt(index,child);
      if(parent.isOnOpenBranch())
      {
         this.refresh(parent.uiIndex);
      }
   }
};
FTreeClass.prototype.getAutoHideScrollBar = function()
{
   return !this.permaScrollBar;
};
FTreeClass.prototype.getDefaultNodeIconSymbolName = function()
{
   return this.defaultNodeIconSymbolName;
};
FTreeClass.prototype.getDrawConnectors = function()
{
   return this.drawConnectors_bool;
};
FTreeClass.prototype.getExpanderSymbolName = function()
{
   return this.expanderSymbolName;
};
FTreeClass.prototype.getExpandNodeTrigger = function()
{
   return this.expandNodeTrigger;
};
FTreeClass.prototype.getNodeAt = function(parent, index)
{
   return parent.getNodeAt(index);
};
FTreeClass.prototype.getTreeLength = function()
{
   var nc = this.rootNode == null ? 0 : this.rootNode.getNumChildren() + 1;
   return nc;
};
FTreeClass.prototype.getRootNode = function()
{
   return this.rootNode;
};
FTreeClass.prototype.getRowCount = function()
{
   return this.numDisplayed;
};
FTreeClass.prototype.getFirstVisibleNode = function()
{
   return this.getItemAt(super.getScrollPosition());
};
FTreeClass.prototype.getSelectedNode = function()
{
   return this.getSelectedItem();
};
FTreeClass.prototype.getSelectedNodes = function()
{
   return this.getSelectedItems();
};
FTreeClass.prototype.getSelectStyle = function()
{
   return this.selectStyle;
};
FTreeClass.prototype.getWidth = function()
{
   return this.width;
};
FTreeClass.prototype.refresh = function()
{
   if(this.enable)
   {
      var brnch = arguments.length <= 0 ? 0 : arguments[0];
      this.dataProvider.updateBranch(brnch);
      this.invalidate("updateControl");
   }
   return true;
};
FTreeClass.prototype.removeNode = function(node)
{
   var p = node.getParentNode();
   if(p == null)
   {
      this.setRootNode(null);
   }
   else
   {
      this.removeNodeAt(p,node.index);
   }
   return true;
};
FTreeClass.prototype.removeNodeAt = function(parentNode, index)
{
   var node = parentNode.getNodeAt(index);
   if(node != null)
   {
      this.removeSelectedAncestors(node);
      parentNode.removeNodeAt(index);
      if(parentNode.isOnOpenBranch())
      {
         this.refresh(parentNode.uiIndex);
      }
      return true;
   }
   return false;
};
FTreeClass.prototype.removeAll = function()
{
   this.setRootNode(null);
   return true;
};
FTreeClass.prototype.setDataProvider = function(dataProvider, labelField, dataField)
{
   if(this.rootNode != null)
   {
      this.rootNode.setDataProvider(dataProvider,labelField,dataField);
      this.refresh();
   }
};
FTreeClass.prototype.setDefaultNodeIconSymbolName = function(name)
{
   this.defaultNodeIconSymbolName = name;
   this.refresh();
   return true;
};
FTreeClass.prototype.setDrawConnectors = function(shouldDraw)
{
   this.drawConnectors_bool = shouldDraw;
   return true;
};
FTreeClass.prototype.setEnabled = function(enabledFlag)
{
   super.setEnabled(enabledFlag);
   this.hsb_mc.setEnabled(enabledFlag);
   this.sb_mc.setEnabled(enabledFlag);
};
FTreeClass.prototype.setExpanderSymbolName = function(symbolName)
{
   this.expanderSymbolName = symbolName;
   return true;
};
FTreeClass.prototype.setNodeExpansionHandler = function(chng, obj)
{
   this.expansionHandlerObj = obj != undefined ? obj : this._parent;
   this.expansionHandlerFxn = chng;
   return true;
};
FTreeClass.prototype.setExpandNodeTrigger = function(trigger)
{
   this.expandNodeTrigger = trigger != "press" ? "release" : trigger;
   this.refresh();
};
FTreeClass.prototype.setRootNode = function(root)
{
   if(root == null)
   {
      delete this.rootNode;
   }
   this.rootNode = root;
   this.clearSelected();
   this.dataProvider.setRootNode(root);
   this.resetTreeWidth();
   this.refresh();
   return true;
};
FTreeClass.prototype.setRowCount = function(count)
{
   var hgt = count * (this.itmHgt - 2) + 2;
   if(this.hsb_mc != undefined)
   {
      hgt += this.hsb_mc._height;
   }
   this.setSize(this.width,hgt);
   return true;
};
FTreeClass.prototype.setFirstVisibleNode = function(node)
{
   if(node.isOnOpenBranch())
   {
      this.setScrollPosition(node.uiIndex);
   }
};
FScrollSelectListClass.prototype.setScrollPosition = function(pos)
{
   if(this.enable)
   {
      pos = Math.min(pos,super.getLength() - super.numDisplayed);
      pos = Math.max(pos,0);
      this.sb_mc.setScrollPosition(pos);
   }
};
FTreeClass.prototype.setSelectedNode = function(node)
{
   this.clearSelected();
   this.selectNode(node,true);
};
FTreeClass.prototype.setSelectedNodes = function(nodesArray)
{
   this.clearSelected();
   var i = 0;
   while(i < nodesArray.length)
   {
      this.selectNode(nodesArray[i],true);
      i++;
   }
};
FTreeClass.prototype.setSelectStyle = function(selectStyle)
{
   this.clearSelected();
   this.selectStyle = selectStyle;
   return true;
};
FTreeClass.prototype.setSize = function(w, h)
{
   if(!this.enable)
   {
      return undefined;
   }
   w = Math.max(w,20);
   this.height = Math.max(h,40);
   this.setNumDisplayed();
   var oldND = this.numDisplayed;
   this.adjNDForHScroll();
   this.height = oldND * (this.itmHgt - 2) + 2;
   if(this.container_mc != undefined)
   {
      this.container_mc.removeMovieClip();
   }
   this.createEmptyMovieClip("container_mc" + this.mcCount,this.mcCount - 2);
   this.container_mc = this["container_mc" + this.mcCount];
   this.mask_mc.removeMovieClip();
   this.attachMovie("FBoundingBoxSymbol","mask_mc",this.mcCount - 1);
   this.mask_mc._width = w;
   this.mask_mc._height = h;
   this.container_mc.setMask(this.mask_mc);
   super.setSize(w,this.height);
};
FTreeClass.prototype.setStyleProperty = function(propName, value, isGlobal)
{
   if(propName != "embedFonts" && (propName.subString(0,4) != "text" || (propName == "textColor" || propName == "textDisabled" || propName == "textFont" || propName == "textSelected" || propName == "textSize")))
   {
      if(propName == "textFont" && value == undefined)
      {
         value = "FTreeDefaultFont";
      }
      super.setStyleProperty(propName,value,isGlobal);
   }
   if(this.drawConnectors_bool || this.expanderSymbolName == undefined)
   {
      this.refresh();
   }
};
FTreeClass.prototype.setWidth = function(w)
{
   this.setSize(w,this.height);
};
FTreeClass.prototype.getSelectedItem = function()
{
   for(var key in this.selected)
   {
      return this.selected[key];
      break;
   }
   return null;
};
FTreeClass.prototype.getSelectedItems = function()
{
   var rtn = new Array();
   for(var key in this.selected)
   {
      rtn.push(this.selected[key]);
   }
   return rtn.length <= 0 ? null : rtn;
};
FTreeClass.prototype.getSelectedIndex = function()
{
   var sel = this.getSelectedItem();
   if(sel != null)
   {
      return sel.uiIndex;
   }
   return null;
};
FTreeClass.prototype.selectItem = function(index, selectedFlag)
{
   this.selectNode(this.getItemAt(index),selectedFlag);
};
FTreeClass.prototype.selectNode = function(node, selectedFlag)
{
   node.isSelected = selectedFlag;
   if(selectedFlag)
   {
      this.selected[String(node.__ID__)] = node;
   }
   else
   {
      delete this.selected[String(node.__ID__)];
   }
   if(this.isNodeVisble(node))
   {
      node.ui.drawItem(node,node.isSelected);
   }
};
FTreeClass.prototype.isSelected = function(index)
{
   return this.getItemAt(index).isSelected;
};
FTreeClass.prototype.clearSelected = function()
{
   for(var key in this.selected)
   {
      this.selected[key].isSelected = false;
      if(this.isNodeVisble(this.selected[key]))
      {
         this.selected[key].ui.drawItem(this.selected[key],false);
      }
   }
   delete this.selected;
   this.selected = new Array();
};
FTreeClass.prototype.removeSelectedAncestors = function(node)
{
   for(var id in this.selected)
   {
      if(node.isAncestor(this.selected[id]))
      {
         delete this.selected[id];
      }
   }
};
FTreeClass.prototype.modelChanged = function(eventObj)
{
   if(eventObj.event == "newRoot")
   {
      this.topDisplayed = 0;
   }
   this.invalidate("updateControl");
};
FTreeClass.prototype.isNodeVisble = function(node)
{
   var tlb = this;
   return node != null && node.isOnOpenBranch() && node.uiIndex >= this.topDisplayed && node.uiIndex < this.topDisplayed + this.numDisplayed;
};
FTreeClass.prototype.moveSelBy = function(incr)
{
   var node = this.getSelectedNode();
   if(node != null && node.isOnOpenBranch())
   {
      super.moveSelBy(incr);
   }
   this.executeCallBack();
};
FTreeClass.prototype.selectionHandler = function(itemnum)
{
   var index = this.topDisplayed + itemNum;
   if(this.getItemAt(index) == undefined)
   {
      this.changeFlag = false;
      return undefined;
   }
   this.changeFlag = true;
   if(this.selectStyle == "single" && !Key.isDown(17) || !Key.isDown(16) && !Key.isDown(17))
   {
      this.clearSelected();
      this.selectItem(index,true);
      this.lastSelected = index;
      this.container_mc["fListItem" + itemNum + "_mc"].drawItem(this.getItemAt(index),this.isSelected(index));
   }
   else if(Key.isDown(16) && this.selectStyle == "multiple")
   {
      if(this.lastSelected == -1)
      {
         this.lastSelected = index;
      }
      var incr = this.lastSelected >= index ? -1 : 1;
      this.clearSelected();
      var i = this.lastSelected;
      while(i != index)
      {
         this.selectItem(i,true);
         var cn = this.getItemAt(i);
         if(this.isNodeVisble(cn))
         {
            this.container_mc["fListItem" + (i - this.topDisplayed) + "_mc"].drawItem(cn,cn.isSelected);
         }
         i += incr;
      }
      this.selectItem(index,true);
      this.container_mc["fListItem" + (index - this.topDisplayed) + "_mc"].drawItem(this.getItemAt(index),this.isSelected(index));
   }
   else if(key.isDown(17))
   {
      var selectedFlag = this.isSelected(index);
      if(this.selectStyle == "single")
      {
         this.clearSelected();
      }
      if(this.selectStyle == "multiple" || !selectedFlag)
      {
         this.selectItem(index,!selectedFlag);
         this.container_mc["fListItem" + itemNum + "_mc"].drawItem(this.getItemAt(this.topDisplayed + itemNum),this.isSelected(this.topDisplayed + itemNum));
      }
      this.lastSelected = index;
   }
};
FTreeClass.prototype.updateControl = function(skipInitSB)
{
   if(this.getLength() <= this.numDisplayed)
   {
      this.topDisplayed = 0;
   }
   var i = 0;
   while(i < this.numDisplayed)
   {
      this.container_mc["fListItem" + i + "_mc"].drawItem(this.getItemAt(this.topDisplayed + i),this.isSelected(this.topDisplayed + i));
      i++;
   }
   if(skipInitSB == undefined || this.hsb_mc == undefined && this.needHS())
   {
      this.initScrollBar();
   }
};
FTreeClass.prototype.needHS = function()
{
   var vsw = this.sb_mc == undefined ? 0 : this.sb_mc._width;
   return this.getTreeWidth() > this.width - vsw;
};
FTreeClass.prototype.initScrollBar = function()
{
   var bAdjustVS = true;
   var bAdjustHS = true;
   if(!this.permaScrollBar && this.getLength() <= this.numDisplayed)
   {
      this.sb_mc.removeMovieClip();
      this.sb_mc = undefined;
      this.scrollOffset = undefined;
      bAdjustVS = false;
      this.mask_mc._width = this.width;
   }
   if(!this.permaScrollBar && !this.needHS())
   {
      if(this.hsb_mc != undefined)
      {
         this.hsb_mc.removeMovieClip();
         this.hsb_mc = undefined;
         this.setSize(this.width,this.height);
      }
      this.hScrollOffset = 0;
      bAdjustHS = false;
   }
   if(bAdjustHS || bAdjustVS)
   {
      if(this.sbc_mc == undefined)
      {
         this.mcCount += 3;
         this.createEmptyMovieClip("sbc_mc" + this.mcCount,this.mcCount);
         this.sbc_mc = this["sbc_mc" + this.mcCount];
      }
      if(bAdjustVS && this.sb_mc == undefined)
      {
         this.attachVSBar();
      }
      if(bAdjustHS && this.hsb_mc == undefined)
      {
         this.sbc_mc.attachMovie("FScrollBarSymbol","hsb_mc",2,{hostStyle:this.styleTable});
         this.hsb_mc = this.sbc_mc.hsb_mc;
         this.hsb_mc.setHorizontal(true);
         this.hsb_mc.setChangeHandler("hScrollHandler",this);
         this.setNumDisplayed();
         this.adjNDForHScroll();
         if(this.sb_mc == undefined && this.getLength() > this.numDisplayed)
         {
            this.attachVSBar();
            bAdjustVS = true;
         }
      }
      if(bAdjustVS)
      {
         var hsh = this.hsb_mc == null ? 0 : this.hsb_mc._height;
         var vh = this.height - hsh;
         this.sb_mc.setSize(vh);
         this.mask_mc._height = vh;
         this.sb_mc._x = this.width - this.sb_mc._width;
         this.sb_mc._y = 0;
         this.sb_mc.setLargeScroll(this.numDisplayed - 1);
         this.scrollOffset = this.sb_mc._width;
         this.sb_mc.setScrollProperties(this.numDisplayed,0,this.getLength() - this.numDisplayed);
      }
      if(bAdjustHS)
      {
         var vsw = this.sb_mc == undefined ? 0 : this.sb_mc._width;
         var vw = this.width - vsw;
         this.mask_mc._width = vw;
         this.hsb_mc.setSize(vw);
         this.hsb_mc._x = 0;
         this.hsb_mc._y = this.height - this.hsb_mc._height;
         this.hsb_mc.setLargeScroll(this.width);
         this.hScrollOffset = this.hsb_mc._width;
         this.hsb_mc.setScrollProperties(this.width,0,this.getTreeWidth() - vw);
         this.hsb_mc.setSmallScroll(5);
      }
   }
};
FTreeClass.prototype.attachVSBar = function()
{
   this.sbc_mc.attachMovie("FScrollBarSymbol","sb_mc",1,{hostStyle:this.styleTable});
   this.sb_mc = this.sbc_mc.sb_mc;
   this.sb_mc.setChangeHandler("scrollHandler",this);
};
FTreeClass.prototype.setNumDisplayed = function()
{
   this.measureItmHgt();
   this.numDisplayed = Math.floor(this.height / (this.itmHgt - 2));
};
FTreeClass.prototype.adjNDForHScroll = function()
{
   if(this.hsb_mc != undefined)
   {
      var totHt = this.numDisplayed * this.itmHgt + this.hsb_mc._height;
      if(totHt > this.height)
      {
         var itmsCvrd = this.hsb_mc._height / this.itmHgt;
         if(itmsCvrd - Math.floor(itmsCvrd) > 0.2)
         {
            itmsCvrd = Math.ceil(itmsCvrd);
            var oldND = this.numDisplayed;
            this.numDisplayed -= itmsCvrd;
            while(itmsCvrd > 0 && this.container_mc != null)
            {
               var ci = oldND - itmsCvrd;
               this.container_mc["fListItem" + ci + "_mc"].removeMovieClip();
               itmsCvrd--;
            }
         }
      }
   }
};
FTreeClass.prototype.scrollHandler = function(scrollBar)
{
   var pos = scrollBar.getScrollPosition();
   this.topDisplayed = pos;
   if(this.lastPosition != pos)
   {
      this.updateControl(true);
   }
   this.lastPosition = pos;
};
FTreeClass.prototype.resetTreeWidth = function()
{
   if(this.getRootNode() != null)
   {
      this.treewidth = this.calcWidth(this.getRootNode());
      this.fixHLs();
   }
   else
   {
      this.treewidth = 0;
   }
   this.invalidate("updateControl");
};
FTreeClass.prototype.setTreeWidth = function(w)
{
   if(w > this.treewidth)
   {
      this.treewidth = w;
      this.fixHLs();
      this.initScrollBar();
   }
};
FTreeClass.prototype.calcWidth = function(node, w)
{
   if(node.width > w)
   {
      w = node.width;
   }
   var i = 0;
   while(node.isOpen() && i < node.length)
   {
      var cw = this.calcWidth(node[i],w);
      if(cw > w)
      {
         w = cw;
      }
      i++;
   }
   return w;
};
FTreeClass.prototype.fixHLs = function()
{
   var i = 0;
   var w = Math.max(this.width,this.treewidth);
   while(i < this.numDisplayed)
   {
      this.container_mc["fListItem" + i + "_mc"].highlight_mc._width = w;
      i++;
   }
};
FTreeClass.prototype.dblClickSpeed = 400;
FTreeClass.prototype.lastClick = 0;
FTreeClass.prototype.lastClicker = undefined;
FTreeClass.prototype.iconDoubleClick = function()
{
   if(this.controller.controller.getEnabled())
   {
      var dbl = false;
      if(this == FTreeClass.prototype.lastClicker)
      {
         if(new Date().getTime() - FTreeClass.prototype.lastClick < FTreeClass.prototype.dblClickSpeed)
         {
            if(this.item.isBranch())
            {
               dbl = true;
               var ex = this.item.ui.cn.ex;
               ex[ex.trigger]();
            }
         }
      }
      if(!dbl)
      {
         this.controller.controller.clickHandler(this.controller.itemNum);
      }
      FTreeClass.prototype.lastClick = new Date().getTime();
      FTreeClass.prototype.lastClicker = this;
   }
};
FTreeClass.prototype.getTreeWidth = function()
{
   return this.treewidth;
};
FTreeClass.prototype.hScrollHandler = function(scrollBar)
{
   var pos = scrollBar.getScrollPosition();
   if(this.lastHPosition != pos)
   {
      this.container_mc._x = - pos;
   }
   this.lastHPosition = pos;
};
FTreeClass.prototype.myOnKeyDown = function()
{
   if(this.focused)
   {
      if(Key.isDown(32))
      {
         var node = this.getSelectedNode();
         if(node != null && node.isBranch())
         {
            var ex = node.ui.cn.ex;
            ex[ex.trigger]();
         }
      }
      else
      {
         super.myOnKeyDown();
      }
   }
};
